#!/usr/libexec/platform-python

import csv
import argparse
import urllib.request, urllib.error, urllib.parse
import json

from intigua_client import IntiguaRestClient


def add_physical_servers(server_list):
    serverExample = { 'ip': '10.10.20.51','name': 'Intigua Splunk Server', 'os': 'Microsoft Windows Server 2008 R2', 'osFullName': 'Microsoft Windows Server 2008 R2', 'dnsname': 'w08r264sp2'}
    try:
        client = IntiguaRestClient(args.user, args.apikey, args.url)
        for id, server in server_list.items():
            if does_server_exist(client,server['name'],server['ip']) == 0:
                client.addServer(server)
                print("Successfully added server "+server['name'])
            else:
                print("Server "+server['name']+" already exists! - Skipping")
    except urllib.error.HTTPError as e:
        if "BadIp" in e.read():
            print("Failed to add " + server['name'] + " -- Bad IP or Problematic credentials (Either: Manager IP, User Name, App-key), exiting...")
            exit(1)
        print("Failed to add "+server['name'] +" -- This might be due to a non-existent operating system")
        print(e)
        print(e.read())
        exit(1)


def does_server_exist(client,server_name,ip):
    server_exists_by_name = client.findServerByParams(hostname=server_name)
    server_exists_by_ip = client.findServerByParams(ip=ip)
    if not server_exists_by_name and not server_exists_by_ip:
        return 0
    else:
        return 1




def csv_file_validator(file_name):
    if not file_name.endswith(".csv"):
        raise argparse.ArgumentTypeError("file should be with .csv extension")
    return file_name


parser = argparse.ArgumentParser(description='Physical Server Bulk Adder')
parser.add_argument('--file', dest='file', type=csv_file_validator, help='Servers CSV Filename', required=True)
parser.add_argument('--url', dest='url', type=str,
                    help='URL to Intigua Server. Please use full rest path. e.g:https://[intigua-servername]/vmanage-server/rest/rest-api',
                    required=True)
parser.add_argument('--user', dest='user', type=str, help='Username', required=True)
parser.add_argument('--apikey', dest='apikey', type=str, help='API Key', required=True)
args = parser.parse_args()

def parse_serverinv_to_dict(serverlist_csv):
    server_list = {}
    try:
        with open(serverlist_csv, 'r') as file_reader:
            dict_reader = csv.DictReader(file_reader)
            i=0
            for server in dict_reader:
                if server ['server name'] != '' and server['ip'] != '' and server['operating system'] != '':
                    server_list[i] = {}
                    server_list[i]['name'] = server['server name']
                    server_list[i]['ip'] = server['ip']
                    server_list[i]['os'] = server['operating system']
                    server_list[i]['osFullName'] = server['operating system']
                    server_list[i]['dnsname'] = server['server name']
                    i += 1
                else:
                    print(server['server name']+" skipped! -- Missing information")
    except IOError as e:
        print(e)
        exit(1)
    return server_list

#def add_physical_servers(server_list):


def main():
    servers_file = args.file
    server_dict = parse_serverinv_to_dict(servers_file)
    add_physical_servers(server_dict)


if __name__ == '__main__':
    main()
