#!/usr/libexec/platform-python

import csv
import argparse
import urllib.request, urllib.error, urllib.parse
import json
import traceback

from intigua_client import IntiguaRestClient
import intigua_client


def add_tag_to_servers(server_list):
    try:
        client = IntiguaRestClient(args.user, args.apikey, args.url)
        for id, server in server_list.items():
            should_remove_flag = False
            s = getServer(client, server['name'])
            new_tags = []
            # print s["tags"]
            for existing_tag in s["tags"]:
                if existing_tag.get("name") == server['tag']:
                    should_remove_flag = True
                    print("Will remove '%s' tag from '%s'" % (server['tag'], server['name']))
                else:
                    new_tags.append(existing_tag)
            if should_remove_flag:
                s["tags"] = new_tags
                client.updateServer(s)
            else:
                 print("Could not find '%s' tag in '%s'" % (server['tag'], server['name']))
            # print s["tags"]
            
    except urllib.error.HTTPError as e:
        print(e)
        print(traceback.format_exc())
        exit(1)

def getServer(client, server):
    l = client.findServer(server)
    if len(l) != 1:
        raise "Server search did not return a single server (%d servers found). Please change your server indetifier to be more specific."
    return l[0]

def does_server_exist(client,server_name,ip):
    server_exists_by_name = client.findServerByParams(hostname=server_name)
    server_exists_by_ip = client.findServerByParams(ip=ip)
    if not server_exists_by_name and not server_exists_by_ip:
        return 0
    else:
        return 1




def csv_file_validator(file_name):
    if not file_name.endswith(".csv"):
        raise argparse.ArgumentTypeError("file should be with .csv extension")
    return file_name


parser = argparse.ArgumentParser(description='Physical Server Bulk Adder')
parser.add_argument('--file', dest='file', type=csv_file_validator, help='Servers CSV Filename', required=True)
parser.add_argument('--url', dest='url', type=str,
                    help='URL to Intigua Server. Please use full rest path. e.g:https://[intigua-servername]/vmanage-server/rest/rest-api',
                    required=True)
parser.add_argument('--user', dest='user', type=str, help='Username', required=True)
parser.add_argument('--apikey', dest='apikey', type=str, help='API Key', required=True)
args = parser.parse_args()

def parse_serverinv_to_dict(serverlist_csv):
    server_list = {}
    try:
        with open(serverlist_csv, 'r') as file_reader:
            dict_reader = csv.DictReader(file_reader)
            i=0
            for server in dict_reader:
                if server ['server name'] != '' and server['tag'] != '':
                    server_list[i] = {}
                    server_list[i]['name'] = server['server name']
                    server_list[i]['tag'] = server['tag']
                    i += 1
                else:
                    print(server['server name']+" skipped! -- Missing information")
    except IOError as e:
        print(e)
        exit(1)
    return server_list

#def add_physical_servers(server_list):


def main():
    servers_file = args.file
    server_dict = parse_serverinv_to_dict(servers_file)
    add_tag_to_servers(server_dict)


if __name__ == '__main__':
    main()
